<?php

namespace Leenooks\SQRL\SQRL;

use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
use Leenooks\SQRL\Models\Nonce;
use Leenooks\SQRL\SQRL;

class Auth
{
	public static function process(array $client_decode,$server_decode,Nonce $sqrl_nonce,int $tif): string
	{
		switch (Arr::get($client_decode,'cmd')) {
			case 'query':
				$response = self::processQuery($client_decode,$server_decode,$sqrl_nonce,$tif);
				break;

			case "ident":
				$response = self::processIdent($client_decode,$server_decode,$sqrl_nonce,$tif);
				break;

			default:
				$response = SQRL\Response::problem($sqrl_nonce->nonce, $tif|SQRL::tifcode('UNSUPPORTED_FUNCTION'), $log_register);
				break;
		}

		return $response;
	}

	/**
	 * SQRL Query from client
	 *
	 * @param array $client_decode
	 * @param string $server_decode
	 * @param Nonce $sqrl_nonce
	 * @param int $tif
	 * @return string
	 */
	protected static function processQuery(array $client_decode,string $server_decode,Nonce $sqrl_nonce,int $tif): string
	{
		$sqrl_pubkey = Pubkey::check(Arr::get($client_decode,'idk'));

		if ($sqrl_pubkey) {
			$sqrl_nonce->pubkey_id = $sqrl_pubkey->id;
			$sqrl_nonce->save();

			Log::debug(__METHOD__,['pubkey'=>$sqrl_pubkey->id,'nonce'=>$sqrl_nonce->id]);

			if ($sqrl_pubkey->disabled == 1) {
				return SQRL\Response::problem($sqrl_nonce->nonce,$tif|SQRL::tifcode('SQRL_DISABLED'));
			}

			//@todo - extra options Arr::get($client_decode,'opt')

			$suk = in_array('suk',Arr::get($client_decode,'opt',[])) ? $sqrl_pubkey->suk : '';

			return Response::respond(
				$sqrl_nonce->nonce,
				$tif|SQRL::tifcode('ID_MATCH'),
				sprintf('%s?nut=%s',config('sqrl.api_route'),$sqrl_nonce->nonce),
				'',
				'',
				0,
				$suk,
				'',
			);
		}

		return Response::respond(
			$sqrl_nonce->nonce,
			$tif,
			sprintf('%s?nut=%s',config('sqrl.api_route'),$sqrl_nonce->nonce),
			'',
			'',
			0,
			'',
			'',
		);
	}

	/**
	 * SQRL Identify request from client
	 *
	 * @param array $client_decode
	 * @param array $server_decode
	 * @param Nonce $sqrl_nonce
	 * @param int $tif
	 * @return string
	 */
	protected static function processIdent(array $client_decode,array $server_decode,Nonce $sqrl_nonce,int $tif): string
	{
		$sqrl_pubkey = Pubkey::check(Arr::get($client_decode,'idk'));

		if (! $sqrl_pubkey) {

			if (! Arr::get($client_decode,'suk') && ! Arr::get($client_decode,'vuk')) {
				return SQRL\Response::problem($sqrl_nonce->nonce,$tif|SQRL::tifcode('CLIENT_FAILURE'));
			}

			$sqrl_pubkey = Pubkey::create(Arr::get($client_decode,'idk'),Arr::get($client_decode,'suk'),Arr::get($client_decode,'vuk'));

			if (! $sqrl_pubkey) {
				return SQRL\Response::problem($sqrl_nonce->nonce,$tif|SQRL::tifcode('TRANSIENT_ERROR'));
			}

		} else {
			$tif |= SQRL::tifcode('ID_MATCH');
		}

		// Check if the user has disabled their key previously.
		if ($sqrl_pubkey->disabled == 1) {
			return SQRL\Response::problem($sqrl_nonce->nonce,$tif|SQRL::tifcode('SQRL_DISABLED'));
		}

		// Link this user to the nonce
		if ((! $sqrl_nonce->pubkey_id) || $sqrl_nonce->pubkey_id !== $sqrl_pubkey->id) {
			$sqrl_nonce->pubkey_id = $sqrl_pubkey->id;
		}

		$sqrl_nonce->verified = 1;
		$sqrl_nonce->save();

		// @todo suk may need to be included in disabled response?
		$suk = in_array('suk',Arr::get($client_decode,'opt',[])) ? $sqrl_pubkey->suk : '';
		$url = in_array('cps',Arr::get($client_decode,'opt',[])) ? SQRL::base64_decode_url($sqrl_nonce->url) : '';

		//@todo - extra options Arr::get($client_decode,'opt')

		return Response::respond(
			$sqrl_nonce->nonce,
			$tif,
			sprintf('%s?nut=%s',config('sqrl.api_route'),$sqrl_nonce->nonce),
			$url,
			'',
			0,
			$suk,
			'',
		);
	}
}
