<?php

namespace Leenooks\SQRL\SQRL;

use Carbon\Carbon;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Request;

use Leenooks\SQRL\Models\Nonce as Model;
use Leenooks\SQRL\SQRL;

class Nonce
{
	protected static $alg = 'sha256';

	/**
	 * Create a new unique nonce
	 *
	 * @param string $source
	 * @return string|null
	 */
	protected static function create(string $source)
	{
		$nonce = NULL;

		do {
			$nonce = hash_hmac(self::$alg,uniqid($source,true),config('sqrl.salt'));
		} while (Model::where('nonce','=',$nonce)->count() > 0);

		return $nonce;
	}

	/**
	 * Create and Store a nonce for auth
	 *
	 * @param string $url
	 * @param string $can
	 * @return Model
	 */
	public static function createAuth(string $url,string $can): Model
	{
		$ip = Request::ip();
		$nonce = static::create($ip.'auth');

		$o = new Model;
		$o->type = 'auth';
		$o->nonce = $nonce;
		$o->orig_nonce = $nonce;
		$o->ip = $ip;
		$o->url = SQRL::base64_encode_url($url);
		$o->can = SQRL::base64_encode_url($can);
		$o->save();

		Log::debug(sprintf('NUT [%s] created for (%s)',$o->nonce,$o->ip));
		return $o;
	}

	/**
	 * Check if our nonce is valid by checking if its in the DB
	 *
	 * @param string $nonce
	 * @return Model|null
	 */
	public static function check(string $nonce,string $key): ?Model
	{
		try {
			$o = Model::where($key,$nonce)->firstOrFail();

		} catch (ModelNotFoundException $e) {
			return NULL;
		}

		// Delete the old nonce
		if ($o->created_at->diffInMinutes(Carbon::now()) > config('sqrl.nonce_age')) {
			$o->delete();

			return NULL;
		}

		return $o;
	}

	/**
	 * During auth, check if our nonce is valid and update it
	 *
	 * @param string $nonce
	 * @return Model|null
	 */
	public static function checkNonceValid(string $nonce): ?Model
	{
		$o = self::check($nonce,'nonce');

		if ($o) {
			$o->nonce = self::create(Request::ip().$o->id.$o->type);

			$o->save();
		}

		return $o;
	}
}
