<?php

namespace Leenooks\SQRL\SQRL;

use Illuminate\Support\Facades\Log;
use Leenooks\SQRL\SQRL;

class Response
{
	/**
	 * Create the HTTP response back to the client
	 *
	 * @param string $response
	 * @return \Illuminate\Http\Response
	 */
	public static function create(string $response): \Illuminate\Http\Response
	{
		$response = response($response,200);

		return $response
			->header('Content-Length',strlen($response->getOriginalContent()))
			->header('Content-Type','application/x-www-form-urlencoded')
			->header('User-Agent','SQRL Server')
			->header('host',config('sqrl.domain'));
	}

	/**
	 * Send back a problem response to the Client
	 *
	 * @param string $nonce
	 * @param int $tif
	 * @return string
	 */
	public static function problem(string $nonce,int $tif): string
	{
		return self::respond(
			$nonce,
			$tif,
			sprintf('%s?nut=%s',config('sqrl.api_route'),$nonce),
			'',
			'',
			0,
			'',
			''
		);
	}

	/**
	 * Send back a response to the Client
	 *
	 * @param string $nonce
	 * @param int $tif
	 * @param string $query
	 * @param string $url
	 * @param string $can
	 * @param int $sin
	 * @param string $suk
	 * @param string $ask
	 * @return string
	 */
	public static function respond(string $nonce,int $tif,string $query,string $url='',string $can='',int $sin=0,string $suk='',string $ask=''): string
	{
		$result = "ver=1\r\n";
		$result .= sprintf("nut=%s\r\n",$nonce);
		$result .= sprintf("tif=%s\r\n",strtoupper(dechex($tif)));
		$result .= sprintf("qry=%s\r\n",$query);

		if ($url)
			$result .= sprintf("url=%s\r\n",$url);

		if ($can)
			$result .= sprintf("can=%s\r\n",$can);

		$result .= sprintf("sin=%d\r\n",$sin);

		if ($suk)
			$result .= sprintf("suk=%s\r\n",SQRL::base64_encode_url($suk));

		if ($ask)
			$result .= sprintf("ask=%s\r\n",$ask);

		Log::debug(sprintf('Response [%s]',str_replace("\r\n",'.',$result)));

		return SQRL::base64_encode_url($result);
	}
}
